/* ************************************************************************
*  File: reports.c                                                        *
*  Usage: Misc. functions for on-line status reports                      *
*  Created by raymond@brokersys.com                                       *
*  Optional displays added by dluther@ex-pressnet.com                     *
***************************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "screen.h"
#include "constants.h"
#include "oasis.h"        /* to avoid repeating NUM_ITEM_WEARS */

/*  Comment out this line if you do not use DGScripts - Note: If you comment
 *  this out, you will get several warnings (5) about declared variables
 *  unused, but everything will work fine. If you have no plan to add
 *  dgscripts, you can go through and weed out all the script related things
 *  if you want to get rid of those warnings. -DHL
 */
#define USEDGSCRIPT 1
/* Comment out this line if you do not use ColorCodes -DHL */
#define USECOLORCODES 1

/*   external vars  */
//extern FILE *player_fl;
extern struct room_data *world;
extern struct char_data *character_list;
extern struct obj_data *object_list;
extern struct obj_data *obj_proto;
extern struct descriptor_data *descriptor_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct zone_data *zone_table;
extern struct char_data *mob_proto;
extern zone_rnum top_of_zone_table;
extern int circle_shutdown, circle_reboot;
extern int circle_restrict;
extern mob_rnum top_of_mobt;
extern obj_rnum top_of_objt;
extern room_rnum top_of_world;
extern const char *wear_bits[];

/* local functions */
int get_weapon_dam(struct obj_data *o);
int get_armor_ac(struct obj_data *o);
void list_object_zone(struct char_data *ch, int whichzone, int switchmode);
void list_mob_zone(struct char_data *ch, int whichzone, int switchmode);

void do_olist(struct char_data *ch, char *input, int switchmode);
void do_mlist(struct char_data *ch, char *input, int switchmode);
void do_rlist(struct char_data *ch, char *input, int switchmode);
void list_wands_staves(struct char_data *ch, char *input);
void list_scrolls_potions(struct char_data *ch, char *input);
void do_list_wear(struct char_data *ch, char *input, int switchmode);

#ifdef USEDGSCRIPT
void quicklist_trigs(char *infobuf, struct script_data *sc);
ACMD(do_tlist);
#endif

/*  These are the definitions for the switch options available.  If you add
 *  a new type, just define it here, put a new abbrev check in the
 *  ACMD(do_wlist) then a new case statement in the appropriate place where
 *  the switch shows up.
 *  -DHL
 */
#define REPORT_NORMAL  0
#define REPORT_SCRIPTS 1
#define REPORT_FLAGS   2
#define REPORT_AFF_X   3
#define REPORT_INFO    4

/*  For each definition above, there should be a matching term in the
 *  const char *reports[] - DHL
 */
const char *reports[] = {
	"",
	"Trigs",
	"Flags",
	"Affs",
	"Info"
};


#ifdef USEDGSCRIPT
/*  This function is used to print out the script information for the output
 *  of the wlist. If you don't use dgscripts, you shouldn't use it for
 *  anything. - DHL
 */
void quicklist_trigs(char *infobuf, struct script_data *sc)
{
  trig_data *t;

  infobuf[0]='\0';

  for (t = TRIGGERS(sc); t; t = t->next) {
    sprintf(infobuf + strlen(infobuf), "%d", GET_TRIG_VNUM(t));
    if (t->next)
      sprintf(infobuf + strlen(infobuf), ",");
  }
  return;
}
#endif

void list_room_zone  (struct char_data *ch, int whichzone, int switchmode)
{
  int i=0;
  int j=0;
  room_vnum first_z_room = 0;        /* vnum of first obj in zone list */
  room_vnum last_z_room;             /* vnum of last obj in zone list */
  int char_zone = ((GET_ROOM_VNUM(ch->in_room))/100);  /*initialize to current zone  */
  char infobuf[MAX_STRING_LENGTH];

  buf[0]='\0';
  if (whichzone > (-1))
    char_zone = whichzone;   /* see if specific zone was passed as arg */

  while (j!= char_zone){
    first_z_room++;
	  j=(world[i].number/100);
	  i++;

    /*  Some zones have 100+ rooms but not 100+ items/objs.  To find the
     *  needed mobs/objects, search backwards for previous zone.
     */
    if (j>char_zone){
 	    if (char_zone>0){
        char_zone--;
	      first_z_room=0;
	      i=0;
	      j=0;
	    } else {
        log("SYSERR:  rlist looked for a zone < 0");
	      return;
	    }
	  }
  }
  first_z_room--;
  last_z_room = first_z_room;

  while (j == char_zone){
    j=(world[i].number/100);
	  last_z_room++;
	  i++;
  }
  if (first_z_room<0)
	first_z_room++;

  for (i = first_z_room ; i < last_z_room ; i++){
    sprintf(buf + strlen(buf), "[%5d]  %-40s", world[i].number, world[i].name);
    switch (switchmode){
#ifdef USEDGSCRIPT
      case REPORT_SCRIPTS:
      // Check for and print out the script specific information. - DHL
  	if (!world[i].script)
          sprintf(infobuf, "None.");
  	else
  	  quicklist_trigs(infobuf, world[i].script);
      break;
#endif
      case REPORT_INFO:
      // Print out the Sector type - DHL
      	sprintf(infobuf, "%s", sector_types[ world[i].sector_type ] );
      break;
      case REPORT_FLAGS:
      // Print out the room flags (PEACEFUL, DARK, etc.) - DHL
	      sprintbit( world[i].room_flags , room_bits, infobuf);
      break;
      default:
	// This should only be called if someone tries an invalid switch for rooms. - DHL
        sprintf(buf + strlen(buf), "\r\n");
      continue;
    }
    // Okay, now append our added data to the original output - DHL
    sprintf(buf + strlen(buf), "  (%s: %s)\r\n", reports[switchmode], infobuf);
  }

  if (strlen(buf)<5) send_to_char ("There are no rooms in the zone you requested.\r\n",ch);
  else page_string (ch->desc,buf,1);
  return;
}

void do_rlist (struct char_data *ch, char *input, int switchmode)
{
  if (!(*input))
    list_room_zone(ch,(-1), 0);
  else {
    skip_spaces(&input);
    if (is_number(input))
	list_room_zone(ch,atoi(input),switchmode);
    else
	list_room_zone(ch,((GET_ROOM_VNUM(ch->in_room)) / 100),switchmode);
  }
  return;
}

int get_weapon_dam (struct obj_data *o)
{
  int i;

  for (i = 0; i < MAX_OBJ_AFFECT; i++)
    if (o->affected[i].location == APPLY_DAMROLL){
	   return (o->affected[i].modifier);
	}
  return (0);
}

int get_armor_ac (struct obj_data *o)
{
  int i;

  for (i = 0 ; i < MAX_OBJ_AFFECT ; i++)
    if (o->affected[i].location == APPLY_AC){
  	  return (o->affected[i].modifier);
	}
  return (0);
}

void list_object_zone  (struct char_data *ch, int whichzone, int switchmode)
{
  int i = 0, j = 0;
  obj_vnum first_z_obj = 0;         /*vnum of first obj in zone list*/
  obj_vnum last_z_obj;              /*vnum of last obj in zone list */
  int char_zone = ((GET_ROOM_VNUM(ch->in_room))/100);  /*initialize to current zone    */
  char infobuf[MAX_STRING_LENGTH], namebuf[80];
  struct obj_data *obj;

  buf[0]='\0';
  if (whichzone > (-1)) char_zone = whichzone;   /*see if specific zone was passed as arg*/

  while (j != char_zone){
    first_z_obj++;
    j=(GET_OBJ_VNUM(&obj_proto[i])/100);
    i++;

    /*some zones have 100+ rooms but not 100+ items/objs.  To find the needed objs/objects,
     * search backwards for previous zones w/them.*/
    if (j>char_zone){
      if (char_zone>0){
        char_zone--;
        first_z_obj=0;
        i=0;
        j=0;
      } else {
		    log("SYSERR:  olist looked for a zone < 0");
		    return;
		  }
		}
	}
  first_z_obj--;
  last_z_obj = first_z_obj;

  while (j == char_zone){
    j=(GET_OBJ_VNUM(&obj_proto[i])/100);
	  last_z_obj++;
	  i++;
	}

  if (first_z_obj<0)
		 first_z_obj++;

  for (i = first_z_obj; i < last_z_obj; i++){
    sprintf(namebuf, "%s", obj_proto[i].short_description);
#ifdef USECOLORCODES
    remove_colorcodes(namebuf);
#endif
    sprintf(buf + strlen(buf), "[%5d] %-32s", GET_OBJ_VNUM(&obj_proto[i]), namebuf);
    switch (switchmode){
      case REPORT_NORMAL:
        j=obj_proto[i].obj_flags.type_flag;
        sprintf(buf + strlen(buf)," %s ", item_types[obj_proto[i].obj_flags.type_flag]);
        if ((j == ITEM_WORN) || (j== ITEM_ARMOR)){
				  sprintf(buf+strlen(buf)," (-%d AC 1) ",GET_OBJ_VAL(&obj_proto[i],0));
				  j=get_armor_ac (&obj_proto[i]);
				  if (j!=0)
						 sprintf(buf + strlen(buf), "(-%d AC 2)",j);
			  }
        if (j==ITEM_WEAPON){
				  sprintf(buf+strlen(buf),"(Ave dam: %.1f) ",
				    (((GET_OBJ_VAL(&obj_proto[i], 2) + 1) / 2.0) * GET_OBJ_VAL(&obj_proto[i], 1)));
				  j  = get_weapon_dam(&obj_proto[i]);
				  if (j!=0)
				  	sprintf(buf+strlen(buf), " +%d DAM ",j);
		    }
        sprintbit(GET_OBJ_WEAR(&obj_proto[i]), wear_bits, buf+strlen(buf));
        sprintf(buf + strlen(buf),"\r\n");
      continue;
 
#ifdef USEDGSCRIPT
      case REPORT_SCRIPTS:
        obj = read_object(i, REAL);
	if (!SCRIPT(obj))
		sprintf(infobuf, "None.");
        else
	  	quicklist_trigs(infobuf, SCRIPT(obj));
        extract_obj(obj);
      break;
#endif

      case REPORT_AFF_X:
        sprintbit(GET_OBJ_PERM(&obj_proto[i]), affected_bits, infobuf);
      break;

      default:
        sprintf(buf + strlen(buf), "\r\n");
      continue;
    }
    sprintf(buf + strlen(buf), "(%s: %s)\r\n", reports[switchmode], infobuf);
  }

  if (strlen(buf)<5)
    send_to_char ("There are no objects in the zone you requested.\r\n",ch);
  else
		page_string (ch->desc,buf,1);

  return;
}

void do_olist (struct char_data *ch, char *input, int switchmode)
{
  if (!(*input))
    list_object_zone(ch,(-1), 0);
  else {
    skip_spaces(&input);
    if (is_number(input))
      list_object_zone(ch,atoi(input), switchmode);
	  else
      list_object_zone(ch,((GET_ROOM_VNUM(ch->in_room)) / 100), switchmode);
  }
  return;
}

void list_mob_zone  (struct char_data *ch, int whichzone, int switchmode)
{
  int i=0, j = 0;
  mob_vnum first_z_mob = 0;             /*vnum of first mob in zone list*/
  mob_vnum last_z_mob;              /*vnum of last mob in zone list */
  int char_zone = ((GET_ROOM_VNUM(ch->in_room))/100);  /*initialize to current zone    */
  char infobuf[MAX_STRING_LENGTH];
  struct char_data *mob;

  buf[0]='\0';
  if (whichzone > (-1))
		char_zone = whichzone;   /*see if specific zone was passed as arg*/

  while (j!= char_zone){
    first_z_mob++;
    j=(GET_MOB_VNUM(&mob_proto[i])/100);
	  i++;

    /*some zones have 100+ rooms but not 100+ items/mobs.  To find the needed mobs/objects,
     * search backwards for previous zonew/them.*/
    if (j>char_zone){
     if (char_zone>0){
       char_zone--;
       first_z_mob=0;
       i=0;
       j=0;
     } else {
       log("SYSERR:  mlist looked for a zone < 0");
	     return;
	   }
	  }
	 }
  first_z_mob --;
  last_z_mob = first_z_mob;

  while (j == char_zone){
    j=(GET_MOB_VNUM(&mob_proto[i])/100);
    last_z_mob++;
    i++; 
	}

  if (first_z_mob<0)
    first_z_mob ++;

  for (i = first_z_mob; i < last_z_mob; i++){
    sprintf(buf + strlen(buf), "[%5d] %-25s ", GET_MOB_VNUM(&mob_proto[i]), mob_proto[i].player.short_descr);
    switch (switchmode){

      case REPORT_NORMAL:
        sprintf(buf + strlen(buf), "(Level: %3d, Exp: %6d)\r\n",
		      mob_proto[i].player.level, 
					mob_proto[i].points.exp);
      continue;

#ifdef USEDGSCRIPT
      case REPORT_SCRIPTS:
        mob = read_mobile(i, REAL);
        char_to_room(mob, 0);
        if (!(SCRIPT(mob)))
          sprintf(infobuf, "None.");
        else
          quicklist_trigs(infobuf, SCRIPT(mob));
        extract_char(mob);
      break;
#endif

      case REPORT_FLAGS:
        sprintbit(MOB_FLAGS(&mob_proto[i]), action_bits, infobuf);
      break;

      case REPORT_AFF_X:
        sprintbit(AFF_FLAGS(&mob_proto[i]), affected_bits, infobuf);
        break;
      default:
        sprintf(buf + strlen(buf), "\r\n");
      continue;
    }
    sprintf(buf + strlen(buf), "(%s: %s)\r\n", reports[switchmode], infobuf);
  }

  if (strlen(buf)<5) send_to_char ("There are no mobs in the zone you requested.\r\n",ch);
  else page_string (ch->desc,buf,1);
  return;
}

void do_mlist (struct char_data *ch, char *input, int switchmode)
{
  if (!(*input))
     list_mob_zone(ch,(-1), 0);
  else {
    skip_spaces(&input);
    if (is_number(input))
		  list_mob_zone(ch,atoi(input), switchmode);
	  else
		  list_mob_zone(ch,((GET_ROOM_VNUM(ch->in_room)) / 100), switchmode);
  }
  return;
}

void list_wands_staves (struct char_data *ch, char *input)
{
  int type;
  int i, j, k;
  char namebuf[80];

  skip_spaces(&input);
  switch (input[0]){
    case 'F':
    case 'f': type = ITEM_STAFF; break;
    case 'W':
    case 'w': type = ITEM_WAND; break;
    default:
      sprintf(buf,"SYSERR:  Default reached in list_scrolls_potions (arg = %s)", input);
      log(buf);
    return;
  }

  buf[0]='\0';
  for (i=0; i < top_of_objt; i++){
    j = obj_proto[i].obj_flags.type_flag;  /*look for specific sort of item*/
    if (j == type){  /*found one*/
      sprintf(namebuf, "%s", obj_proto[i].short_description);
#ifdef USECOLORCODES
      remove_colorcodes(namebuf);
#endif
      sprintf(buf+strlen(buf),"[%5d] %-30s", GET_OBJ_VNUM(&obj_proto[i]), namebuf);
   		/*
		    values 0-3:
		    Potion, Scroll - up to three spells [values 1-3]
	   	*/
  	  sprintf(buf+strlen(buf), " ( Spell: ");
	    for (k=1; k < 4; k++){
        if ((GET_OBJ_VAL(&obj_proto[i], k)) != (-1))
			    sprintf(buf+strlen(buf), "%s ", skill_name(GET_OBJ_VAL(&obj_proto[i], k)));
	    }
	  sprintf(buf+strlen(buf), ")\r\n");
    } /*if j == type*/
	}  /*for i...*/
  page_string (ch->desc, buf, 1);
}

void list_scrolls_potions (struct char_data *ch, char *input)
{
  int type, i, j, k;
  char namebuf[80];

  skip_spaces(&input);
  switch (input[0]){
    case 'S':
    case 's': type = ITEM_SCROLL; break;
    case 'P':
    case 'p': type = ITEM_POTION; break;
    default :
	    sprintf(buf,"SYSERR:  Default reached in list_scrolls_potions (arg = %s)", input);
      log (buf);
	  return;
  }  /*switch...*/

  buf[0]='\0';
  for (i=0; i < top_of_objt; i++){
	  j = obj_proto[i].obj_flags.type_flag;  /*look for specific sort of item*/
	  if (j == type){  /*found one*/
      sprintf(namebuf, "%s", obj_proto[i].short_description);
#ifdef USECOLORCODES
      remove_colorcodes(namebuf);
#endif
      sprintf(buf+strlen(buf),"[%5d] %-20s", GET_OBJ_VNUM(&obj_proto[i]), namebuf);
    /*  values 0-3:
     *  Potion, Scroll - up to three spells [values 1-3]
     */
    sprintf(buf+strlen(buf), " ( Spells ");
    for (k=1; k < 4; k++){
        if ((GET_OBJ_VAL(&obj_proto[i], k)) != (-1))
          sprintf(buf+strlen(buf), "-%s ",
	      skill_name(GET_OBJ_VAL(&obj_proto[i], k)));
	  }
      sprintf(buf+strlen(buf), ")\r\n");
	  } /*if j == type*/
	}  /*for i...*/
  page_string (ch->desc, buf, 1);
}

void do_list_wear (struct char_data *ch, char *input, int switchmode)
{
  char j = atoi(input);
  int i, found = 0;
  char namebuf[80];
  char infobuf[MAX_STRING_LENGTH];
  struct obj_data *obj;
 
  if (input[0] == '?'){
	   j=0;
	   send_to_char ("Wear positions:\r\n", ch);
	   for (i = 0; i < NUM_ITEM_WEARS; i++){
      sprintf(buf, "(%2d) %-20.20s %s", i + 1, wear_bits[i], (++j % 2) ? "\r\n" : "");
      send_to_char(buf, ch);
	   }
	   send_to_char("\r\nIf you choose TAKE, you will be shown item that are !Take\r\n",ch);
	   return;
  }

  buf[0]='\0';
  j--; /*to be used with NAMES array*/

  if (j==0){  /*Show ony !Take items for this option*/
  	for (i=0; i < top_of_objt; i++)   /*cycle through every obj*/
	   if (!(CAN_WEAR(&obj_proto[i], (1<<j)))) {  /*check exact bit for requested position*/
       sprintf(namebuf, "%s", obj_proto[i].short_description);
#ifdef USECOLORCODES
       remove_colorcodes(namebuf);
#endif
       sprintf(buf+strlen(buf),"[%5d] %-32s !TAKE",
				       GET_OBJ_VNUM(&obj_proto[i]), namebuf);
#ifdef USEDGSCRIPT
         if (switchmode == 1){
           obj = read_object(i, REAL);
           if (!SCRIPT(obj))
             sprintf(infobuf, "None.");
           else
             quicklist_trigs(infobuf, SCRIPT(obj));
           sprintf(buf + strlen(buf), " (Trigs: %s)\r\n", infobuf);
           extract_obj(obj);
         } else
           sprintf(buf + strlen(buf), "\r\n");
#else
         sprintf(buf + strlen(buf), "\r\n");
#endif
	     }
	 page_string (ch->desc, buf, 1);
	 return;
      }

    for (i=0;i<top_of_objt;i++){   /*cycle through every obj*/
	    if (CAN_WEAR(&obj_proto[i], (1<<j))){  /*check exact bit for requested position*/
        sprintf(namebuf, "%s", obj_proto[i].short_description);
#ifdef USECOLORCODES
        remove_colorcodes(namebuf);
#endif
        sprintf(buf+strlen(buf),"[%5d] %-32s %s",
          GET_OBJ_VNUM(&obj_proto[i]), namebuf, wear_bits[j]);
#ifdef USEDGSCRIPT
        if (switchmode == REPORT_SCRIPTS){
          obj = read_object(i, REAL);
          if (!SCRIPT(obj))
            sprintf(infobuf, "None.");
          else
            quicklist_trigs(infobuf, SCRIPT(obj));
          sprintf(buf + strlen(buf), " (Trigs: %s)\r\n", infobuf);
          extract_obj(obj);
        } else
          sprintf(buf + strlen(buf), "\r\n");
#else
        sprintf(buf + strlen(buf), "\r\n");          
#endif
        found = 1;
	    }
	  }

    if (!*buf)
      send_to_char("There are no items of that type in the object files.c\r\n",ch);
    else if (found == 0)
	  	send_to_char("There are no itmes of that type.\r\n", ch);
    page_string (ch->desc, buf, 1);
}


ACMD (do_wlist)
{
  char *message="Usage: wlist < o | m | s | t | h | w | p | e | r | # (?)> <zone> <switch>\r\nType HELP WLIST for details\r\n";
  int char_zone = ((GET_ROOM_VNUM(ch->in_room))/100);
  char switchtype[MAX_INPUT_LENGTH];
  int switchmode;

  argument = two_arguments(argument, buf, buf2);
  one_argument(argument, switchtype);

  if (!*buf)
   {
     send_to_char(message,ch);
     return;
    }

  /*  If you add more switch types, just put in the abbrev check for it
   *  here and assign it a definition at the top of the file.
   */
  if (!*switchtype)
     switchmode = 0;
#ifdef USEDGSCRIPTS
  else if (is_abbrev(switchtype, "scripts"))
     switchmode = REPORT_SCRIPTS;
#endif
  else if (is_abbrev(switchtype, "affs"))
     switchmode = REPORT_AFF_X;
  else if (is_abbrev(switchtype, "flags"))
     switchmode = REPORT_FLAGS;
  else if (is_abbrev(switchtype, "info"))
  	 switchmode = REPORT_INFO;
  else
	 switchmode = REPORT_NORMAL;

  switch (buf[0])
    {
      case 'O':
      case 'o':
	do_olist(ch, buf2, switchmode);
	break;
      case 'M':
      case 'm':
	do_mlist(ch, buf2, switchmode);
	break;
      case 'F':
      case 'f':
      case 'W':
      case 'w':
        list_wands_staves(ch,buf);
        break;
      case 'P':
      case 'p':
      case 'S':
      case 's':
        list_scrolls_potions(ch,buf);
        break;
      case 'R':
      case 'r':
	do_rlist(ch,buf2, switchmode);
	break;
#ifdef USEHEDIT
      case 'H':
      case 'h':
        /* insert wlist help entries here */
        break;
#endif
#ifdef USEDGSCRIPT
      case 'T':
      case 't':
	if (!*buf2)
          sprintf(buf2, "%d", char_zone);
	do_tlist(ch, buf2, 0, 0);
	break;
#endif
      case '#':
	do_list_wear(ch, buf2, switchmode);
	break;
      default:
	send_to_char(message, ch);
	break;
    }  /*switch...*/
  return;
}
